import 'package:flutter/material.dart';

/// This package helps you show elegant transitions between your screens.

/// Animation types
enum AnimationType {
  normal,
  fade,
  scale,
  rotation,
  slideRightToLeft,
  slideLeftToRight,
  slideUpToDown,
  slideDownToUp
}

/// Main class for transitions route
class NavigatorTransitionsRoute {
  /// Context is de BuildContext of the application at that moment.
  final BuildContext context;

  /// Child is the Widget to navigate.
  final Widget child;

  /// Animation type for the transition
  final AnimationType animation;

  /// Time duration for the transition.
  final Duration duration;

  /// Use if you want to replacement push
  final bool replacement;

  NavigatorTransitionsRoute(
      {@required this.context,
      @required this.child,
      this.replacement = false,
      this.animation = AnimationType.normal,
      this.duration = const Duration(milliseconds: 200)}) {
    switch (this.animation) {
      case AnimationType.normal:
        this._normalTransition();
        break;
      case AnimationType.fade:
        this._fadeTransition();
        break;
      case AnimationType.scale:
        this._scaleTransition();
        break;
      case AnimationType.rotation:
        this._rotationTransition();
        break;
      case AnimationType.slideRightToLeft:
        this._slideRTLTransition();
        break;
      case AnimationType.slideLeftToRight:
        this._slideLTRTransition();
        break;
      case AnimationType.slideUpToDown:
        this._slideUpToDownTransition();
        break;
      case AnimationType.slideDownToUp:
        this._slideDownToUpTransition();
        break;
    }
  }

  /// Push normal navigator of page
  void _pushPage(Route route) => Navigator.push(context, route);

  /// Push replacement navigator of page
  void _pushReplacementPage(Route route) =>
      Navigator.pushReplacement(context, route);

  /// Controller of normal transition
  void _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => this.child);

    (this.replacement)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }

  /// Controller of fade transition
  void _fadeTransition() {
    final route = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.child,
        transitionDuration: duration,
        transitionsBuilder: (_, animation, __, child) {
          return FadeTransition(
              child: child,
              opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
                  CurvedAnimation(parent: animation, curve: Curves.easeOut)));
        });

    (this.replacement)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }

  /// Controller of scale transition
  void _scaleTransition() {
    final route = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.child,
        transitionDuration: duration,
        transitionsBuilder: (_, animation, __, child) {
          return ScaleTransition(
              child: child,
              scale: Tween<double>(begin: 0.0, end: 1.0).animate(
                  CurvedAnimation(parent: animation, curve: Curves.easeOut)));
        });

    (this.replacement)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }

  /// Controller of rotation transition
  void _rotationTransition() {
    final route = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.child,
        transitionDuration: duration,
        transitionsBuilder: (_, animation, __, child) {
          return RotationTransition(
              child: child,
              turns: Tween<double>(begin: 0.0, end: 1.0).animate(
                  CurvedAnimation(parent: animation, curve: Curves.easeOut)));
        });

    (this.replacement)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }

  /// Controller of slide right to left transition
  void _slideRTLTransition() {
    final route = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.child,
        transitionDuration: duration,
        transitionsBuilder: (_, animation, __, child) {
          final tween = Tween(begin: Offset(1.0, 0.0), end: Offset.zero)
              .chain(CurveTween(curve: Curves.easeOut));

          return SlideTransition(
              child: child, position: animation.drive(tween));
        });

    (this.replacement)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }

  /// Controller of slide left to right transition
  void _slideLTRTransition() {
    final route = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.child,
        transitionDuration: duration,
        transitionsBuilder: (_, animation, __, child) {
          final tween = Tween(begin: Offset(-1.0, 0.0), end: Offset.zero)
              .chain(CurveTween(curve: Curves.easeOut));

          return SlideTransition(
              child: child, position: animation.drive(tween));
        });

    (this.replacement)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }

  /// Controller of slite up to down transition
  void _slideUpToDownTransition() {
    final route = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.child,
        transitionDuration: duration,
        transitionsBuilder: (_, animation, __, child) {
          final tween = Tween(begin: Offset(0.0, -1.0), end: Offset.zero)
              .chain(CurveTween(curve: Curves.easeOut));

          return SlideTransition(
              child: child, position: animation.drive(tween));
        });

    (this.replacement)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }

  /// Controller of down to up transition
  void _slideDownToUpTransition() {
    final route = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.child,
        transitionDuration: duration,
        transitionsBuilder: (_, animation, __, child) {
          final tween = Tween(begin: Offset(0.0, 1.0), end: Offset.zero)
              .chain(CurveTween(curve: Curves.easeOut));

          return SlideTransition(
              child: child, position: animation.drive(tween));
        });

    (this.replacement)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }
}
